const HELP_GROUP = "Help options:";
const CONFIG_GROUP = "Config options:";
const BASIC_GROUP = "Basic options:";
const OUTPUT_GROUP = "Output options:";
const ADVANCED_GROUP = "Advanced options:";
const DISPLAY_GROUP = "Stats options:";

module.exports = {
	commands: [
		{
			name: "create",
			alias: "c",
			scope: "external",
			type: String,
			description: "Initialize a new webpack configuration"
		},
		/* TODO: tbd{
			name: "build",
			alias: "b",
			scope: "external",
			type: String,
			description: "Build a project"
		}, */
		{
			name: "migrate",
			alias: "m",
			scope: "external",
			type: String,
			description: "Migrate a configuration to a new version"
		},
		{
			name: "add",
			scope: "external",
			alias: "a",
			type: String,
			description: "Add a property to your configuration"
		},
		{
			name: "remove",
			scope: "external",
			alias: "r",
			type: String,
			description: "Remove a property from your configuration"
		},
		{
			name: "update",
			alias: "u",
			scope: "external",
			type: String,
			description: "Update a property in your webpack configuration"
		},
		/* TODO: Dev Server needs PR to fix flags once released {
			name: "serve",
			alias: "s",
			type: String,
			scope: "external",
			description: "Start a development server with webpack"
		}, */
		{
			name: "loader",
			scope: "external",
			alias: "l",
			type: String,
			description: "Scaffold a loader repository"
		},
		{
			name: "plugin",
			alias: "p",
			scope: "external",
			type: String,
			description: "Scaffold a plugin repository"
		},
		{
			name: "info",
			scope: "external",
			type: String,
			description: "Outputs information about your system and dependencies"
		},
		{
			name: "help",
			scope: 'internal',
			type: Boolean,
			description: "Outputs the list of arguments",
		},
		{
			name: "version",
			scope: 'internal',
			type: Boolean,
			description: "Get current version"
		}
		/* TODO: tbd
		{
			name: "interactive",
			scope: "internal",
			type: String,
			alias: "i",
			description: "Use webpack interactively"
		},
		{
			name: "analyze",
			type: Boolean,
			description:
				"Analyze webpack for performance"
		}, */
	],
	core: [
		{
			name: "entry",
			type: String,
			multiple: true,
			defaultValue: null,
			defaultOption: true,
			group: BASIC_GROUP,
			description: "The entry point of your application"
		},
		{
			name: "config",
			alias: "c",
			type: String,
			defaultValue: null,
			group: CONFIG_GROUP,
			description: "Path to a webpack configuration file",
		},
		{
			name: "register",
			type: String,
			multiple: true,
			alias: "r",
			group: CONFIG_GROUP,
			description:
				"Preload one or more modules before loading the webpack configuration"
		},
		{
			name: "merge",
			alias: "m",
			type: String,
			group: CONFIG_GROUP,
			description: "Merge a configuration file",
		},
		{
			name: "progress",
			type: String,
			defaultValue: "bar",
			group: BASIC_GROUP,
			description: "Print compilation progress in percentage",
		},
		{
			name: "colors",
			type: Boolean,
			defaultValue: true,
			group: DISPLAY_GROUP,
			description: "Enables/Disables colors on the console",
		},
/*  TODO: add logging levels		{
			name: "quiet",
			alias: "q",
			type: Boolean,
			group: DISPLAY_GROUP,
			description: "Suppress any output",
		},
		{
			name: "verbose",
			alias: "v",
			type: Boolean,
			group: DISPLAY_GROUP,
			description: "Show more details",
			typeLabel: '{underline ms}'
		}, */
		{
			name: "help",
			type: Boolean,
			group: HELP_GROUP,
			description: "Outputs the list of arguments",
		},
		{
			name: "output",
			alias: "o",
			group: OUTPUT_GROUP,
			type: String,
			description: "The output path where webpack will generate files",
		},
		{
			name: "filename",
			alias: "f",
			group: OUTPUT_GROUP,
			type: String,
			description: "Name the output file generated by webpack",
		},
		{
			name: "global",
			alias: "g",
			type: String,
			multiple: true,
			group: ADVANCED_GROUP,
			description: "Declares and exposes a global variable.",
		},
		{
			name: "target",
			alias: "t",
			defaultValue: 'web',
			type: String,
			group: ADVANCED_GROUP,
			description: "Sets the build target",
		},
		{
			name: "watch",
			type: Boolean,
			alias: "w",
			group: BASIC_GROUP,
			description: "Watch for files changes.",
		},
		{
			name: "hot",
			alias: "h",
			type: Boolean,
			group: ADVANCED_GROUP,
			description: "Enables Hot Module Replacement.",
		},
		{
			name: "debug",
			type: Boolean,
			group: ADVANCED_GROUP,
			description: "Switch loaders to debug mode",
		},
		{
			name: "sourcemap",
			type: String,
			alias: "s",
			defaultValue: "eval",
			group: BASIC_GROUP,
			description: "Determine source maps to use",
		},
		{
			name: "prefetch",
			type: String,
			multiple: true,
			group: ADVANCED_GROUP,
			description: "Prefetch this request",
		},
		{
			name: "json",
			type: Boolean,
			alias: "j",
			description: "Prints result as JSON.",
			group: DISPLAY_GROUP,
		},
		// TBD set defaults
		{
			name: "dev",
			alias: "d",
			type: Boolean,
			defaultValue: false,
			group: BASIC_GROUP,
			description: "Run development build",
		},
		// TBD set defaults
		{
			name: "prod",
			alias: "p",
			type: Boolean,
			defaultValue: true,
			group: BASIC_GROUP,
			description: "Run production build",
		},
		{
			name: "version",
			type: Boolean,
			group: BASIC_GROUP,
			description: "Get current version"
		}
	]
};
